@@ -53,10 +53,12 @@ ALL_ZONE_STEREO = "ALL ZONE STEREO"
 
 SOUND_MODE_MAPPING = OrderedDict(
     [('MUSIC', ['PLII MUSIC', 'DTS NEO:6 MUSIC', 'DOLBY D +NEO:X M',
-                'DTS NEO:X MUSIC', 'DOLBY PL2 MUSIC']),
+                'DTS NEO:X MUSIC', 'DOLBY PL2 MUSIC', 'AURO-3D',
+                'AURO-2D SURROUND']),
      ('MOVIE', ['PLII MOVIE', 'PLII CINEMA', 'DTS NEO:X CINEMA',
                 'DTS NEO:6 CINEMA', 'DOLBY D +NEO:X C',
-                'PLIIX CINEMA', 'DOLBY PLII MOVIE']),
+                'PLIIX CINEMA', 'DOLBY PLII MOVIE', 'AURO-3D',
+                'AURO-2D SURROUND']),
      ('GAME', ['PLII GAME', 'DOLBY D +NEO:X G']),
      ('AUTO', ['None']),
      ('STANDARD', ['None2']),
@@ -87,7 +89,8 @@ SOUND_MODE_MAPPING = OrderedDict(
                        'DTS + NEURAL:X', 'MULTI CH IN', 'DTS-HD MSTR',
                        'DTS VIRTUAL:X', 'DTS-HD + NEURAL:X', 'DTS-HD',
                        'DTS + VIRTUAL:X']),
-     ('MCH STEREO', ['MULTI CH STEREO', 'MULTI_CH_STEREO', 'MCH STEREO']),
+     ('MCH STEREO', ['MULTI CH STEREO', 'MULTI_CH_STEREO', 'MCH STEREO',
+                     'MULTI CH IN 7.1']),
      ('STEREO', ['STEREO']),
      (ALL_ZONE_STEREO, ['ALL ZONE STEREO'])])
 
@@ -321,6 +324,13 @@ class DenonAVR:
         self._frequency = None
         self._station = None
 
+        self._tone_control_status = None
+        self._tone_control_adjust = None
+        self._bass = None
+        self._bass_level = None
+        self._treble = None
+        self._treble_level = None
+
         # Get initial setting of values
         self.update()
         # Create instances of additional zones if requested
@@ -388,7 +398,7 @@ class DenonAVR:
                     _LOGGER.error(
                         "Host %s returned malformed XML for end point %s: %s",
                         self._host, command, err)
-                raise ValueError
+                raise ValueError from err
         else:
             if not suppress_errors:
                 _LOGGER.error((
@@ -635,95 +645,106 @@ class DenonAVR:
         Returns "True" on success and "False" on fail.
         This method is for AVR-X  devices built in 2016 and later.
         """
-        success = True
-        # Collect tags for AppCommand.xml call
-        tags = ["GetAllZonePowerStatus", "GetAllZoneSource",
-                "GetAllZoneVolume", "GetAllZoneMuteStatus",
-                "GetSurroundModeStatus"]
-        # Execute call
-        try:
-            root = self.exec_appcommand_post(tags)
-        except requests.exceptions.ConnectTimeout:
-            if compatibiliy_check is True:
-                return None
-            # Assume device is off on connect timeout
-            self._power = POWER_OFF
-            self._state = STATE_OFF
-            return False
-        # Check result
-        if root is None:
-            if compatibiliy_check is False:
-                _LOGGER.error("Update failed.")
-            return False
+        # Use ThreadPoolExecutor to call all URLs of this method in parallel
+        with ThreadPoolExecutor(max_workers=2) as executor:
 
-        # Extract relevant information
-        zone = self._get_own_zone()
+            success = True
+            # Collect tags for AppCommand.xml call
+            tags = ["GetAllZonePowerStatus", "GetAllZoneSource",
+                    "GetAllZoneVolume", "GetAllZoneMuteStatus",
+                    "GetSurroundModeStatus"]
+            # Execute call
+            app_command = executor.submit(self.exec_appcommand_post, tags)
 
-        try:
-            self._power = root[0].find(zone).text
-        except (AttributeError, IndexError):
-            if compatibiliy_check is False:
-                _LOGGER.error("No PowerStatus found for zone %s", self.zone)
-            success = False
+            # Update tone control
+            executor.submit(self._update_tone_control)
 
-        try:
-            self._mute = root[3].find(zone).text
-        except (AttributeError, IndexError):
-            if compatibiliy_check is False:
-                _LOGGER.error("No MuteStatus found for zone %s", self.zone)
-            success = False
-
-        try:
-            self._volume = root.find(
-                "./cmd/{zone}/volume".format(zone=zone)).text
-        except AttributeError:
-            if compatibiliy_check is False:
-                _LOGGER.error("No VolumeStatus found for zone %s", self.zone)
-            success = False
-
-        try:
-            inputfunc = root.find(
-                "./cmd/{zone}/source".format(zone=zone)).text
-        except AttributeError:
-            if compatibiliy_check is False:
-                _LOGGER.error("No Source found for zone %s", self.zone)
-            success = False
-        else:
             try:
-                self._input_func = self._input_func_list_rev[inputfunc]
-            except KeyError:
-                _LOGGER.info("No mapping for source %s found", inputfunc)
-                self._input_func = inputfunc
-                # Get/update sources list if current source is not known yet
-                if self._update_input_func_list():
-                    _LOGGER.info("List of input functions refreshed.")
-                    # If input function is still not known, log error.
-                    if (inputfunc not in self._input_func_list and
-                            inputfunc is not None):
-                        _LOGGER.error(
-                            "Input function %s is not known", self._input_func)
-                else:
-                    _LOGGER.error((
-                        "Input function list for Denon receiver at host %s "
-                        "could not be updated."), self._host)
-        try:
-            self._sound_mode_raw = root[4][0].text.rstrip()
-        except (AttributeError, IndexError):
-            if compatibiliy_check is False:
-                _LOGGER.error(
-                    "No SoundMode found for the main zone %s", self.zone
-                )
-            success = False
-
-        # Now playing information is not implemented for 2016+ models, because
-        # a HEOS API query needed. So only sync the power state for now.
-        if self._receiver_type == AVR_X_2016.type:
-            if self._power == POWER_ON:
-                self._state = STATE_ON
-            else:
+                root = app_command.result()
+            except requests.exceptions.ConnectTimeout:
+                if compatibiliy_check is True:
+                    return None
+                # Assume device is off on connect timeout
+                self._power = POWER_OFF
                 self._state = STATE_OFF
+                return False
+            # Check result
+            if root is None:
+                if compatibiliy_check is False:
+                    _LOGGER.error("Update failed.")
+                return False
 
-        return success
+            # Extract relevant information
+            zone = self._get_own_zone()
+
+            try:
+                self._power = root[0].find(zone).text
+            except (AttributeError, IndexError):
+                if compatibiliy_check is False:
+                    _LOGGER.error(
+                        "No PowerStatus found for zone %s", self.zone)
+                success = False
+
+            try:
+                self._mute = root[3].find(zone).text
+            except (AttributeError, IndexError):
+                if compatibiliy_check is False:
+                    _LOGGER.error("No MuteStatus found for zone %s", self.zone)
+                success = False
+
+            try:
+                self._volume = root.find(
+                    "./cmd/{zone}/volume".format(zone=zone)).text
+            except AttributeError:
+                if compatibiliy_check is False:
+                    _LOGGER.error(
+                        "No VolumeStatus found for zone %s", self.zone)
+                success = False
+
+            try:
+                inputfunc = root.find(
+                    "./cmd/{zone}/source".format(zone=zone)).text
+            except AttributeError:
+                if compatibiliy_check is False:
+                    _LOGGER.error("No Source found for zone %s", self.zone)
+                success = False
+            else:
+                try:
+                    self._input_func = self._input_func_list_rev[inputfunc]
+                except KeyError:
+                    _LOGGER.info("No mapping for source %s found", inputfunc)
+                    self._input_func = inputfunc
+                    # Update sources list if current source is not known yet
+                    if self._update_input_func_list():
+                        _LOGGER.info("List of input functions refreshed.")
+                        # If input function is still not known, log error.
+                        if (inputfunc not in self._input_func_list and
+                                inputfunc is not None):
+                            _LOGGER.error(
+                                "Input function %s is not known",
+                                self._input_func)
+                    else:
+                        _LOGGER.error((
+                            "Input function list for Denon receiver at host %s"
+                            " could not be updated."), self._host)
+            try:
+                self._sound_mode_raw = root[4][0].text.rstrip()
+            except (AttributeError, IndexError):
+                if compatibiliy_check is False:
+                    _LOGGER.error(
+                        "No SoundMode found for the main zone %s", self.zone
+                    )
+                success = False
+
+            # Now playing information is not implemented for 2016+ models, as
+            # a HEOS API query needed. So only sync the power state for now.
+            if self._receiver_type == AVR_X_2016.type:
+                if self._power == POWER_ON:
+                    self._state = STATE_ON
+                else:
+                    self._state = STATE_OFF
+
+            return success
 
     def _update_input_func_list(self):
         """
@@ -827,15 +848,15 @@ class DenonAVR:
             except requests.exceptions.ConnectTimeout:
                 pass
             except (ValueError, requests.exceptions.RequestException):
-                _LOGGER.warning("Receiver name could not be determined. "
-                                "Using standard name: Denon AVR.")
+                _LOGGER.info("Receiver name could not be determined. "
+                             "Using standard name: Denon AVR.")
                 self._name = "Denon AVR"
             else:
                 # Get the tags from this XML
                 name_tag = self._get_status_from_xml_tags(root, name_tag)
                 if name_tag:
-                    _LOGGER.warning("Receiver name could not be determined. "
-                                    "Using standard name: Denon AVR.")
+                    _LOGGER.info("Receiver name could not be determined. "
+                                 "Using standard name: Denon AVR.")
                     self._name = "Denon AVR"
 
     def _get_receiver_name_avr_2016(self):
@@ -871,6 +892,43 @@ class DenonAVR:
         else:
             return self._get_support_sound_mode_avr()
 
+    def _update_tone_control(self):
+        """Update tone control related things."""
+        try:
+            root = self.exec_appcommand_post(attribute_list=['GetToneControl'])
+        except requests.exceptions.ConnectTimeout:
+            root = None
+
+        if root is None:
+            _LOGGER.error("Getting tone control failed.")
+            return False
+
+        try:
+            self._tone_control_status = bool(int(root[0].find('status').text))
+        except (AttributeError, IndexError, TypeError):
+            return False
+
+        if self._tone_control_status is False:
+            # Tone Control can not be activated.
+            # e.g.: Due to active DynamicEQ
+            self._tone_control_adjust = None
+            self._bass = None
+            self._bass_level = None
+            self._treble = None
+            self._treble_level = None
+            return False
+
+        try:
+            self._tone_control_adjust = bool(int(root[0].find('adjust').text))
+            self._bass = int(root[0].find('bassvalue').text)
+            self._bass_level = root[0].find('basslevel').text
+            self._treble = int(root[0].find('treblevalue').text)
+            self._treble_level = root[0].find('treblelevel').text
+        except (AttributeError, IndexError, TypeError):
+            _LOGGER.error("Incomplete/no information found for tone control")
+            return False
+        return True
+
     def _get_support_sound_mode_avr(self):
         """
         Get if sound mode is supported from device.
@@ -1557,6 +1615,26 @@ class DenonAVR:
         """Indicate if all inputs are shown or just active one."""
         return self._show_all_inputs
 
+    @property
+    def bass(self):
+        """Return value of bass."""
+        return self._bass
+
+    @property
+    def bass_level(self):
+        """Return level of bass."""
+        return self._bass_level
+
+    @property
+    def treble(self):
+        """Return value of treble."""
+        return self._treble
+
+    @property
+    def treble_level(self):
+        """Return level of treble."""
+        return self._treble_level
+
     @input_func.setter
     def input_func(self, input_func):
         """Setter function for input_func to switch input_func of device."""
@@ -1788,7 +1866,7 @@ class DenonAVR:
                 return False
 
     def power_on(self):
-        """Turn off receiver via HTTP get command."""
+        """Turn on receiver via HTTP get command."""
         try:
             if self.send_get_command(self._urls.command_power_on):
                 self._power = POWER_ON
@@ -1865,6 +1943,109 @@ class DenonAVR:
             _LOGGER.error("Connection error: mute command not sent.")
             return False
 
+    def _set_tone_control_command(self, parameter_type, value):
+        """Post request for tone control commands."""
+        root = ET.Element("tx")
+        ET.SubElement(root, 'cmd', id='1').text = 'SetToneControl'
+        ET.SubElement(root, parameter_type).text = str(value)
+        tree = ET.ElementTree(root)
+
+        body = BytesIO()
+        tree.write(body, encoding="utf-8", xml_declaration=True)
+
+        try:
+            self.send_post_command(
+                command=self._urls.appcommand, body=body.getvalue()
+                )
+        except (
+                requests.exceptions.ConnectTimeout,
+                requests.exceptions.RequestException
+                ):
+            _LOGGER.error("No connection to %s end point on host %s",
+                          self._urls.appcommand, self._host)
+            return False
+        return True
+
+    def enable_tone_control(self):
+        """Enable tone control to change settings like bass or treble."""
+        if self._tone_control_status is False:
+            return False
+
+        if self._tone_control_adjust is True:
+            return True
+        elif self._set_tone_control_command(parameter_type='adjust', value=1):
+            self._tone_control_adjust = True
+            return True
+        return False
+
+    def disable_tone_control(self):
+        """Disable tone control to change settings like bass or treble."""
+        if self._tone_control_status is False:
+            return False
+
+        if self._tone_control_adjust is False:
+            return True
+        elif self._set_tone_control_command(parameter_type='adjust', value=0):
+            self._tone_control_adjust = False
+            return True
+        return False
+
+    def _set_tone_control(self, parameter_type, value):
+        """Set tone control parameter."""
+        if value < 0 or value > 12:
+            raise ValueError(
+                "Invalid value for {parameter_type}".format(
+                    parameter_type=parameter_type
+                    )
+            )
+
+        if not self.enable_tone_control():
+            return False
+
+        if self._set_tone_control_command(
+                parameter_type='{parameter_type}value'.format(
+                    parameter_type=parameter_type
+                    ), value=value
+                ):
+            setattr(
+                self,
+                '_{parameter_type}'.format(parameter_type=parameter_type),
+                value
+                )
+            setattr(
+                self,
+                '_{parameter_type}_level'.format(
+                    parameter_type=parameter_type
+                    ),
+                '{value:{sign}}dB'.format(
+                    value=value-6, sign='' if value-6 == 0 else '+'
+                    )
+                )
+            return True
+        return False
+
+    def set_bass(self, bass):
+        """
+        Set receiver bass.
+
+        Minimum is 0, maximum at 12
+
+        Note:
+        Doesn't work, if Dynamic Equalizer is active.
+        """
+        return self._set_tone_control(parameter_type='bass', value=bass)
+
+    def set_treble(self, treble):
+        """
+        Set receiver treble.
+
+        Minimum is 0, maximum at 12
+
+        Note:
+        Doesn't work, if Dynamic Equalizer is active.
+        """
+        return self._set_tone_control(parameter_type='treble', value=treble)
+
 
 class DenonAVRZones(DenonAVR):
     """Representing an additional zone of a Denon AVR Device."""
